"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseConfig = parseConfig;
exports.createPlaceholder = exports.getOutputAndPublicPath = void 0;

var _path = _interopRequireDefault(require("path"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const MIMES = {
  jpg: "image/jpeg",
  jpeg: "image/jpeg",
  png: "image/png",
  webp: "image/webp"
};
const EXTS = {
  "image/jpeg": "jpg",
  "image/png": "png",
  "image/webp": "webp"
};

function parseConfig(loaderContext, config, defaults) {
  const outputContext = config.context || loaderContext.rootContext || loaderContext.options && loaderContext.options.context;
  const outputPlaceholder = Boolean(config.placeholder) || defaults.outputPlaceholder;
  const placeholderSize = parseInt(config.placeholderSize, 10) || defaults.placeholderSize; // JPEG and WEBP compression

  const quality = parseInt(config.quality, 10) || defaults.quality; // Useful when converting from PNG to JPG

  const background = config.background;
  let mime;
  let ext;

  if (config.format) {
    mime = MIMES[config.format];
    ext = EXTS[mime];
  } else {
    ext = _path.default.extname(loaderContext.resourcePath).replace(/\./, "");
    mime = MIMES[ext];
  }

  const name = (config.name || defaults.name).replace(/\[ext\]/gi, ext);
  const min = config.min !== undefined ? parseInt(config.min, 10) : undefined;
  const max = config.max !== undefined ? parseInt(config.max, 10) : undefined;
  const steps = config.steps === undefined ? defaults.steps : parseInt(config.steps, 10);
  let generatedSizes;

  if (typeof min === "number" && max) {
    generatedSizes = [];

    for (let step = 0; step < steps; step++) {
      const size = min + (max - min) / (steps - 1) * step;
      generatedSizes.push(Math.ceil(size));
    }
  }

  const esModule = config.esModule !== undefined ? config.esModule : defaults.esModule;
  const emitFile = config.emitFile !== undefined ? config.emitFile : defaults.emitFile;
  return {
    outputContext,
    outputPlaceholder,
    placeholderSize,
    quality,
    background,
    ext,
    mime,
    name,
    generatedSizes,
    esModule,
    emitFile
  };
}

const createPlaceholder = ({
  data
}, mime) => {
  return `"data:${mime};base64,${data.toString("base64")}"`;
};
/**
 * **Responsive Images Loader Paths**
 *
 * Returs the output and public path
 *
 * @method getOutputAndPublicPath
 *
 * @param {string} fileName
 * @param {Config} outputPath
 * @param {Config} publicPath
 *
 * @return {Config} Paths Result
 */


exports.createPlaceholder = createPlaceholder;

const getOutputAndPublicPath = (fileName, {
  outputPath: configOutputPath,
  publicPath: configPublicPath
}) => {
  let outputPath = fileName;

  if (configOutputPath) {
    if (typeof configOutputPath === "function") {
      outputPath = configOutputPath(fileName);
    } else {
      outputPath = _path.default.posix.join(configOutputPath, fileName);
    }
  }

  let publicPath = `__webpack_public_path__ + ${JSON.stringify(outputPath)}`;

  if (configPublicPath) {
    if (typeof configPublicPath === "function") {
      publicPath = configPublicPath(fileName);
    } else if (configPublicPath.endsWith("/")) {
      publicPath = configPublicPath + fileName;
    } else {
      publicPath = `${configPublicPath}/${fileName}`;
    }

    publicPath = JSON.stringify(publicPath);
  }

  return {
    outputPath,
    publicPath
  };
};

exports.getOutputAndPublicPath = getOutputAndPublicPath;